@@ -9,10 +9,18 @@ from django.utils.timezone import now
 from allianceauth.notifications import notify
 from allianceauth.services.hooks import get_extension_logger
 from allianceauth.services.tasks import QueueOnce
+from app_utils.esi import retry_task_if_esi_is_down
 from app_utils.logging import LoggerAddTag
 
 from . import __title__
-from .models import DiscordWebhook, NotificationRule, ScheduledNotification, Timer
+from .models import (
+    DiscordWebhook,
+    DistancesFromStaging,
+    NotificationRule,
+    ScheduledNotification,
+    StagingSystem,
+    Timer,
+)
 
 logger = LoggerAddTag(get_extension_logger(__name__), __title__)
 TASK_PRIO_HIGH = 4
@@ -119,7 +127,10 @@ def notify_about_new_timer(timer_pk: int, notification_rule_pk: int) -> None:
         )
     else:
         if notification_rule.is_enabled and notification_rule.webhook.is_enabled:
-            content = f"New timer added by **{timer.eve_character}**:"
+            author_text = (
+                f" by **{timer.eve_character}**" if timer.eve_character else ""
+            )
+            content = f"New timer added{author_text}:"
             timer.send_notification(
                 webhook=notification_rule.webhook,
                 content=notification_rule.prepend_ping_text(content),
@@ -180,7 +191,7 @@ def schedule_notifications_for_timer(timer_pk: int, is_new: bool = False) -> Non
                     NotificationRule.objects.select_related("webhook")
                     .filter(
                         is_enabled=True,
-                        trigger=NotificationRule.TRIGGER_NEW_TIMER_CREATED,
+                        trigger=NotificationRule.Trigger.NEW_TIMER_CREATED,
                         webhook__is_enabled=True,
                     )
                     .conforms_with_timer(timer)
@@ -198,15 +209,13 @@ def schedule_notifications_for_timer(timer_pk: int, is_new: bool = False) -> Non
             # trigger: timer elapses soon
             with transaction.atomic():
                 # remove existing scheduled notifications if date has changed
-                for obj in ScheduledNotification.objects.filter(timer=timer).exclude(
-                    timer_date=timer.date
-                ):
+                for obj in timer.scheduled_notifications.exclude(timer_date=timer.date):
                     _revoke_notification_for_timer(scheduled_notification=obj)
 
                 # schedule new notifications
                 for notification_rule in NotificationRule.objects.filter(
                     is_enabled=True,
-                    trigger=NotificationRule.TRIGGER_SCHEDULED_TIME_REACHED,
+                    trigger=NotificationRule.Trigger.SCHEDULED_TIME_REACHED,
                 ).conforms_with_timer(timer):
                     _schedule_notification_for_timer(
                         timer=timer, notification_rule=notification_rule
@@ -228,7 +237,7 @@ def schedule_notifications_for_rule(notification_rule_pk: int) -> None:
             notification_rule_pk,
         )
     else:
-        if notification_rule.trigger == NotificationRule.TRIGGER_NEW_TIMER_CREATED:
+        if notification_rule.trigger == NotificationRule.Trigger.NEW_TIMER_CREATED:
             logger.error(
                 "NotificationRule with pk = %s has the wrong trigger. Aborting.",
                 notification_rule_pk,
@@ -239,7 +248,7 @@ def schedule_notifications_for_rule(notification_rule_pk: int) -> None:
                 notification_rule,
             )
             with transaction.atomic():
-                for obj in notification_rule.schedulednotification_set.filter(
+                for obj in notification_rule.scheduled_notifications.filter(
                     timer_date__gt=now()
                 ):
                     _revoke_notification_for_timer(scheduled_notification=obj)
@@ -297,3 +306,48 @@ def housekeeping() -> None:
     logger.info("Performing housekeeping")
     deleted_count = Timer.objects.delete_obsolete()
     logger.info(f"Deleted {deleted_count:,} obsolete timers.")
+
+
+@shared_task
+def calc_staging_system(staging_system_pk: int, force_update: bool = False) -> None:
+    """Recalc distances from a staging system for all timers."""
+    for timer_pk in Timer.objects.values_list("pk", flat=True):
+        calc_timer_distances_for_staging_system.delay(
+            timer_pk, staging_system_pk, force_update
+        )
+
+
+@shared_task
+def calc_timer_distances_for_all_staging_systems(
+    timer_pk: int, force_update: bool = False
+) -> None:
+    """Recalc distances for a timer from a staging system."""
+    timer = Timer.objects.get(pk=timer_pk)
+    for staging_system_pk in StagingSystem.objects.values_list("pk", flat=True):
+        calc_timer_distances_for_staging_system.apply_async(
+            kwargs={
+                "timer_pk": timer.pk,
+                "staging_system_pk": staging_system_pk,
+                "force_update": force_update,
+            },
+            priority=TASK_PRIO_HIGH,
+        )
+
+
+@shared_task(
+    bind=True,
+    max_retries=3,
+    autoretry_for=(OSError,),
+    retry_kwargs={"max_retries": 3},
+    retry_backoff=30,
+)
+def calc_timer_distances_for_staging_system(
+    self, timer_pk: int, staging_system_pk: int, force_update: bool = False
+) -> None:
+    """Calc distances for a timer from a staging system."""
+    retry_task_if_esi_is_down(self)
+    timer = Timer.objects.get(pk=timer_pk)
+    staging_system = StagingSystem.objects.get(pk=staging_system_pk)
+    DistancesFromStaging.objects.calc_timer_for_staging_system(
+        timer=timer, staging_system=staging_system, force_update=force_update
+    )
